==> deadbolt-0.3.1.tgz: package/README.md <==
# deadbolt

## Description

Having locks in async world is not so straight-forward as it may look like.
Everything can work fine, but when exception will be thrown in your app - locks
may stay active until script's termination.

Here comes *deadbolt* to save all us:

```javascript
var deadbolt = require('deadbolt').create();

deadbolt.lock('some-action-id', function (err, lock) {

  if (err) console.error(err); // 'some-action-id' is already running somewhere

  doAsyncActionThatProbablyThrows(function () {
    lock.release(function (err) {
      // Released lock if err === null
    });
  });

}).autorelease(function (err) {

  console.error('Execution stopped somewhere inside lock');
  console.error('Released lock automatically');
  console.error('Reason: ' + err);

});
```

## How does it work?

It's using [v8](https://github.com/v8/v8) [MakeWeak](http://bespin.cz/~ondras/html/classv8_1_1Persistent.html#ab04609812113450bece2640ad0b27658)
black magic. When `lock` becomes weak (not referenced in your program) -
`autorelease` callback will be called.



#### LICENSE

(The MIT License)

Copyright (c) 2011 Fedor Indutny <fedor@indutny.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the 'Software'), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

==> deadbolt-0.3.7.tgz: package/README.md <==
# deadbolt

## Description

Having locks in async world is not so straight-forward as it may look like.
Everything can work fine, but when exception will be thrown in your app - locks
may stay active until script's termination.

Here comes *deadbolt* to save all us:

```javascript
var deadbolt = require('deadbolt');

deadbolt.lock('some-action-id', function (err, lock) {

  if (err) console.error(err); // 'some-action-id' is already running somewhere

  doAsyncActionThatProbablyThrows(function () {
    lock.release(function (err) {
      // Released lock if err === null
    });
  });

}).autorelease(function (err) {

  console.error('Execution stopped somewhere inside lock');
  console.error('Released lock automatically');
  console.error('Reason: ' + err);

});
```

Or just wrap callbacks without setting lock:

```javascript
function asyncAction(callback) {
  callback = deadbolt.wrap(callback);
  doOtherAsyncActionThatMayForgotToCall(callback);
};
```

## Installation

If you're using npm:

```bash
npm install deadbolt
```

From source:

```bash
git clone git://github.com/indutny/deadbolt.git
cd deadbolt
./configure && make
```

## How does it work?

It's using [v8](https://github.com/v8/v8)'s [MakeWeak](http://bespin.cz/~ondras/html/classv8_1_1Persistent.html#ab04609812113450bece2640ad0b27658)
black magic. When `lock` becomes weak (not referenced in your program) -
`autorelease` callback will be called.



#### LICENSE

(The MIT License)

Copyright (c) 2011 Fedor Indutny <fedor@indutny.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the 'Software'), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
